/*
 * blanco Framework
 * Copyright (C) 2004-2009 IGA Tosiki
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 */
/*******************************************************************************
 * Copyright (c) 2009 IGA Tosiki, NTT DATA BUSINESS BRAINS Corp.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    IGA Tosiki (NTT DATA BUSINESS BRAINS Corp.) - initial API and implementation
 *******************************************************************************/
package blanco.commons.calc.parser.concretesax;

import java.io.OutputStream;
import java.io.Writer;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;

import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

;

/**
 * BlancoCalcParserDefSerializerۉSAXNX<br>
 * SAX𗘗pXMLobsONXłB<br>
 * ̃NX͉͗pXMLt@C͂ƂċۉSAXNXƂĐ܂B<br>
 * ̃\[XR[hblancoIgɂ@BIɎĂ܂B ̃NX implements ContentHandler ͍s܂B<br>
 * T^Iȗp@͉L̂悤ɂȂ܂B<br>
 * 
 * <pre>
 * BlancoCalcParserDefSerializer serializer = new BlancoCalcParserDefSerializer(
 *         outStream);
 * serializer.startDocument();
 * serializer.startElementXXXX();
 * serializer.characters(&quot;&quot;);
 * serializer.endElementXXXX();
 * serializer.endDocument();
 * </pre>
 */
public class BlancoCalcParserDefSerializer {
    /**
     * Iɗpo͗pSAXnh<br>
     * ÃXg[͊OcloseKv܂B
     */
    private TransformerHandler _saxHandler;

    public BlancoCalcParserDefSerializer(OutputStream outStream)
            throws TransformerConfigurationException {
        TransformerFactory tf = TransformerFactory.newInstance();
        SAXTransformerFactory saxTf = (SAXTransformerFactory) tf;
        _saxHandler = saxTf.newTransformerHandler();
        _saxHandler.setResult(new StreamResult(outStream));
    }

    public BlancoCalcParserDefSerializer(Writer writer)
            throws TransformerConfigurationException {
        TransformerFactory tf = TransformerFactory.newInstance();
        SAXTransformerFactory saxTf = (SAXTransformerFactory) tf;
        _saxHandler = saxTf.newTransformerHandler();
        _saxHandler.setResult(new StreamResult(writer));
    }

    /**
     * startDocument𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void startDocument() throws SAXException {
        _saxHandler.startDocument();
    }

    /**
     * endDocument𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void endDocument() throws SAXException {
        _saxHandler.endDocument();
    }

    /** startPrefixMapping𔭐܂B */
    public void startPrefixMapping(String prefix, String uri)
            throws SAXException {
        _saxHandler.startPrefixMapping(prefix, uri);
    }

    /**
     * endPrefixMapping𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void endPrefixMapping(String prefix) throws SAXException {
        _saxHandler.endPrefixMapping(prefix);
    }

    /**
     * setDocumentLocator𔭐܂BB
     * 
     * @param locator
     *            P[^
     */
    public void setDocumentLocator(Locator locator) {
        _saxHandler.setDocumentLocator(locator);
    }

    /**
     * processingInstruction𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void processingInstruction(String target, String data)
            throws SAXException {
        _saxHandler.processingInstruction(target, data);
    }

    /**
     * skippedEntity𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void skippedEntity(String name) throws SAXException {
        _saxHandler.skippedEntity(name);
    }

    /**
     * characters\bhĂяo܂B<br>
     * 
     * @param ch
     *            o͂
     * @param start
     *            Jnʒu
     * @param length
     *            ̒
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void characters(char[] ch, int start, int length)
            throws SAXException {
        _saxHandler.characters(ch, start, length);
    }

    /**
     * characters\bhĂяo܂B<br>
     * ȈՉ̂߂  java.lang.StrinĝłB
     * 
     * @param data
     *            o͂
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void characters(String data) throws SAXException {
        final char[] chars = data.toCharArray();
        _saxHandler.characters(chars, 0, chars.length);
    }

    /**
     * ignorableWhitespace𔭐܂B
     * 
     * @throws SAXException
     *             SAX֘A̗OꍇB
     */
    public void ignorableWhitespace(char[] ch, int start, int length)
            throws SAXException {
        _saxHandler.ignorableWhitespace(ch, start, length);
    }

    /**
     * startElementړt̏C[blanco]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [blanco]<br>
     * ړt̏C[blanco]<br>
     * 
     * @param attrVersion
     *            Agr[g[version]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementBlanco(String attrVersion) throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrVersion != null) {
            attributes.addAttribute("", "version", "version", "CDATA",
                    attrVersion);
        }
        _saxHandler.startElement("", "blanco", "blanco", attributes);
    }

    /**
     * endElementړt̏C[blanco]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [blanco]<br>
     * ړt̏C[blanco]<br>
     */
    public void endElementBlanco() throws SAXException {
        _saxHandler.endElement("", "blanco", "blanco");
    }

    /**
     * startElementړt̏C[target]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [target]<br>
     * ړt̏C[target]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementTarget(String attrName) throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        _saxHandler.startElement("", "target", "target", attributes);
    }

    /**
     * endElementړt̏C[target]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [target]<br>
     * ړt̏C[target]<br>
     */
    public void endElementTarget() throws SAXException {
        _saxHandler.endElement("", "target", "target");
    }

    /**
     * startElementړt̏C[blancocalcparser]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [blancocalcparser]<br>
     * ړt̏C[blancocalcparser]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementBlancocalcparser(String attrName)
            throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        _saxHandler.startElement("", "blancocalcparser", "blancocalcparser",
                attributes);
    }

    /**
     * endElementړt̏C[blancocalcparser]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [blancocalcparser]<br>
     * ړt̏C[blancocalcparser]<br>
     */
    public void endElementBlancocalcparser() throws SAXException {
        _saxHandler.endElement("", "blancocalcparser", "blancocalcparser");
    }

    /**
     * startElementړt̏C[propertyblock]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [propertyblock]<br>
     * ړt̏C[propertyblock]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     * @param attrWaitY
     *            Agr[g[waitY]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementPropertyblock(String attrName, String attrWaitY)
            throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        if (attrWaitY != null) {
            attributes.addAttribute("", "waitY", "waitY", "CDATA", attrWaitY);
        }
        _saxHandler.startElement("", "propertyblock", "propertyblock",
                attributes);
    }

    /**
     * endElementړt̏C[propertyblock]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [propertyblock]<br>
     * ړt̏C[propertyblock]<br>
     */
    public void endElementPropertyblock() throws SAXException {
        _saxHandler.endElement("", "propertyblock", "propertyblock");
    }

    /**
     * startElementړt̏C[startstring]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [startstring]<br>
     * ړt̏C[startstring]<br>
     */
    public void startElementStartstring() throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        _saxHandler.startElement("", "startstring", "startstring", attributes);
    }

    /**
     * endElementړt̏C[startstring]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [startstring]<br>
     * ړt̏C[startstring]<br>
     */
    public void endElementStartstring() throws SAXException {
        _saxHandler.endElement("", "startstring", "startstring");
    }

    /**
     * startElementړt̏C[propertykey]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [propertykey]<br>
     * ړt̏C[propertykey]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     * @param attrWaitX
     *            Agr[g[waitX]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementPropertykey(String attrName, String attrWaitX)
            throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        if (attrWaitX != null) {
            attributes.addAttribute("", "waitX", "waitX", "CDATA", attrWaitX);
        }
        _saxHandler.startElement("", "propertykey", "propertykey", attributes);
    }

    /**
     * endElementړt̏C[propertykey]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [propertykey]<br>
     * ړt̏C[propertykey]<br>
     */
    public void endElementPropertykey() throws SAXException {
        _saxHandler.endElement("", "propertykey", "propertykey");
    }

    /**
     * startElementړt̏C[value]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [value]<br>
     * ړt̏C[value]<br>
     */
    public void startElementValue() throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        _saxHandler.startElement("", "value", "value", attributes);
    }

    /**
     * endElementړt̏C[value]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [value]<br>
     * ړt̏C[value]<br>
     */
    public void endElementValue() throws SAXException {
        _saxHandler.endElement("", "value", "value");
    }

    /**
     * startElementړt̏C[valuemapping]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [valuemapping]<br>
     * ړt̏C[valuemapping]<br>
     */
    public void startElementValuemapping() throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        _saxHandler
                .startElement("", "valuemapping", "valuemapping", attributes);
    }

    /**
     * endElementړt̏C[valuemapping]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [valuemapping]<br>
     * ړt̏C[valuemapping]<br>
     */
    public void endElementValuemapping() throws SAXException {
        _saxHandler.endElement("", "valuemapping", "valuemapping");
    }

    /**
     * startElementړt̏C[result]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [result]<br>
     * ړt̏C[result]<br>
     */
    public void startElementResult() throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        _saxHandler.startElement("", "result", "result", attributes);
    }

    /**
     * endElementړt̏C[result]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [result]<br>
     * ړt̏C[result]<br>
     */
    public void endElementResult() throws SAXException {
        _saxHandler.endElement("", "result", "result");
    }

    /**
     * startElementړt̏C[source]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [source]<br>
     * ړt̏C[source]<br>
     */
    public void startElementSource() throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        _saxHandler.startElement("", "source", "source", attributes);
    }

    /**
     * endElementړt̏C[source]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [source]<br>
     * ړt̏C[source]<br>
     */
    public void endElementSource() throws SAXException {
        _saxHandler.endElement("", "source", "source");
    }

    /**
     * startElementړt̏C[tableblock]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [tableblock]<br>
     * ړt̏C[tableblock]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     * @param attrWaitY
     *            Agr[g[waitY]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     * @param attrTitleheight
     *            Agr[g[titleheight]̒ln܂BAgr[gZbgȂꍇɂ
     *            nullZbgĂB
     * @param attrRowname
     *            Agr[g[rowname]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementTableblock(String attrName, String attrWaitY,
            String attrTitleheight, String attrRowname) throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        if (attrWaitY != null) {
            attributes.addAttribute("", "waitY", "waitY", "CDATA", attrWaitY);
        }
        if (attrTitleheight != null) {
            attributes.addAttribute("", "titleheight", "titleheight", "CDATA",
                    attrTitleheight);
        }
        if (attrRowname != null) {
            attributes.addAttribute("", "rowname", "rowname", "CDATA",
                    attrRowname);
        }
        _saxHandler.startElement("", "tableblock", "tableblock", attributes);
    }

    /**
     * endElementړt̏C[tableblock]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [tableblock]<br>
     * ړt̏C[tableblock]<br>
     */
    public void endElementTableblock() throws SAXException {
        _saxHandler.endElement("", "tableblock", "tableblock");
    }

    /**
     * startElementړt̏C[tablecolumn]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [tablecolumn]<br>
     * ړt̏C[tablecolumn]<br>
     * 
     * @param attrName
     *            Agr[g[name]̒ln܂BAgr[gZbgȂꍇɂ nullZbgĂB
     */
    public void startElementTablecolumn(String attrName) throws SAXException {
        AttributesImpl attributes = new AttributesImpl();
        if (attrName != null) {
            attributes.addAttribute("", "name", "name", "CDATA", attrName);
        }
        _saxHandler.startElement("", "tablecolumn", "tablecolumn", attributes);
    }

    /**
     * endElementړt̏C[tablecolumn]ƂČĂяo܂B<br>
     * {Iȏ̓\bhɊ܂܂̂ň͏Ă܂B<br>
     * OURI []<br>
     * [J [tablecolumn]<br>
     * ړt̏C[tablecolumn]<br>
     */
    public void endElementTablecolumn() throws SAXException {
        _saxHandler.endElement("", "tablecolumn", "tablecolumn");
    }
}
